@@ -30,11 +30,12 @@ class AutoSlugField(SlugField):
     AutoSlugField can also perform the following tasks on save:
 
     - populate itself from another field (using `populate_from`),
-    - use custom `slugify` function (can be defined in :doc:`settings`), and
+    - use custom `slugify` function (using `slugify` or :doc:`settings`), and
     - preserve uniqueness of the value (using `unique` or `unique_with`).
 
-    None of the tasks is mandatory, i.e. you can have auto-populated non-unique fields,
-    manually entered unique ones (absolutely unique or within a given date) or both.
+    None of the tasks is mandatory, i.e. you can have auto-populated non-unique
+    fields, manually entered unique ones (absolutely unique or within a given
+    date) or both.
 
     Uniqueness is preserved by checking if the slug is unique with given constraints
     (`unique_with`) or globally (`unique`) and adding a number to the slug to make
@@ -42,9 +43,14 @@ class AutoSlugField(SlugField):
 
     :param populate_from: string or callable: if string is given, it is considered
         as the name of attribute from which to fill the slug. If callable is given,
-        it should accept `instance` parameter and return a value to fill the slug with.
-    :param unique: boolean: ensure total slug uniqueness (unless more precise `unique_with`
-        is defined).
+        it should accept `instance` parameter and return a value to fill the slug
+        with.
+    :param sep: string: if defined, overrides default separator for automatically
+        incremented slug index (i.e. the "-" in "foo-2").
+    :param slugify: callable: if defined, overrides `AUTOSLUG_SLUGIFY_FUNCTION`
+        defined in :doc:`settings`.
+    :param unique: boolean: ensure total slug uniqueness (unless more precise
+        `unique_with` is defined).
     :param unique_with: string or tuple of strings: name or names of attributes
         to check for "partial uniqueness", i.e. there will not be two objects
         with identical slugs if these objects share the same values of given
@@ -53,7 +59,8 @@ class AutoSlugField(SlugField):
         slug, however, may reappear on another date. If more than one field is
         given, e.g. ``unique_with=('pub_date', 'author')``, then the same slug may
         reappear within a day or within some author's articles but never within
-        a day for the same author.
+        a day for the same author. Foreign keys are also supported, i.e. not only
+        `unique_with='author'` will do, but also `unique_with='author__name'`.
 
     .. note:: always place any slug attribute *after* attributes referenced
         by it (i.e. those from which you wish to `populate_from` or check
@@ -95,6 +102,14 @@ class AutoSlugField(SlugField):
         # ensure that slug is unique with given date AND category
         slug = AutoSlugField(unique_with=('pub_date','category'))
 
+        # ensure that slug in unique with an external object
+        # assuming that author=ForeignKey(Author)
+        slug = AutoSlugField(unique_with='author')
+
+        # ensure that slug in unique with a subset of external objects (by lookups)
+        # assuming that author=ForeignKey(Author)
+        slug = AutoSlugField(unique_with='author__name')
+
         # mix above-mentioned behaviour bits
         slug = AutoSlugField(populate_from='title', unique_with='pub_date')
 
@@ -107,6 +122,12 @@ class AutoSlugField(SlugField):
         # autoslugify value from a custom callable
         # (ex. usage: user profile models)
         slug = AutoSlugField(populate_from=lambda instance: instance.user.get_full_name())
+
+        # autoslugify value using custom `slugify` function
+        from autoslug.settings import slugify as default_slugify
+        def custom_slugify(value):
+            return default_slugify(value).replace('-', '_')
+        slug = AutoSlugField(slugify=custom_slugify)
     """
     def __init__(self, *args, **kwargs):
         kwargs['max_length'] = kwargs.get('max_length', 50)
@@ -121,6 +142,11 @@ class AutoSlugField(SlugField):
         if isinstance(self.unique_with, basestring):
             self.unique_with = (self.unique_with,)
 
+        self.slugify = kwargs.pop('slugify', slugify)
+        assert hasattr(self.slugify, '__call__')
+
+        self.index_sep = kwargs.pop('sep', SLUG_INDEX_SEPARATOR)
+
         # backward compatibility
         if kwargs.get('unique_with_date'):
             warn('Using unique_with_date="foo" in AutoSlugField is deprecated, '\
@@ -148,7 +174,7 @@ class AutoSlugField(SlugField):
                 print 'Failed to populate slug %s.%s from %s' % \
                     (instance._meta.object_name, self.name, self.populate_from)
 
-        slug = slugify(value)
+        slug = self.slugify(value)
 
         if not slug:
             # no incoming value,  use model name
@@ -184,30 +210,29 @@ class AutoSlugField(SlugField):
         names) was specified for the field, all these fields are included together
         in the query when looking for a "rival" model instance.
         """
-        def _get_lookups(instance):
+        base_instance = instance
+
+        def _get_lookups(instance, unique_with):
             "Returns a dict'able tuple of lookups to ensure slug uniqueness"
-            for _field_name in self.unique_with:
-                # `inner` is the 'month' part in 'pub_date__month'.
-                # it *only* applies to dates, otherwise it's just ignored.
+            for _field_name in unique_with:
                 if '__' in _field_name:
-                    if _field_name.count('__') > 1:
-                        raise ValueError('The `unique_with` constraint in %s.%s'
-                                         ' is set to "%s", but AutoSlugField only'
-                                         ' accepts one level of nesting for dates'
-                                         ' (e.g. "date__month").'
-                                         % (instance._meta.object_name, self.name,
-                                            _field_name))
-                    field_name, inner = _field_name.split('__')
+                    field_name, inner = _field_name.split('__', 1)
                 else:
                     field_name, inner = _field_name, None
 
+                if not hasattr(instance, '_meta'):
+                    raise ValueError('Could not resolve lookup "...%s" in %s.%s'
+                                     ' `unique_with`.'
+                                     % (_field_name, base_instance._meta.object_name,
+                                        self.name))
+
                 try:
                     field = instance._meta.get_field(field_name)
                 except FieldDoesNotExist:
                     raise ValueError('Could not find attribute %s.%s referenced'
                                      ' by %s.%s (see constraint `unique_with`)'
                                      % (instance._meta.object_name, field_name,
-                                        instance._meta.object_name, self.name))
+                                        base_instance._meta.object_name, self.name))
 
                 value = getattr(instance, field_name)
                 if not value:
@@ -216,11 +241,20 @@ class AutoSlugField(SlugField):
                                      ' Please ensure that "%s" is declared *after*'
                                      ' all fields it depends on (i.e. "%s"), and'
                                      ' that they are not blank.'
-                                     % (instance._meta.object_name, self.name,
+                                     % (base_instance._meta.object_name, self.name,
                                         instance._meta.object_name, field_name,
                                         self.name, '", "'.join(self.unique_with)))
                 if isinstance(field, DateField):    # DateTimeField is a DateField subclass
                     inner = inner or 'day'
+
+                    if '__' in inner:
+                        raise ValueError('The `unique_with` constraint in %s.%s'
+                                         ' is set to "%s", but AutoSlugField only'
+                                         ' accepts one level of nesting for dates'
+                                         ' (e.g. "date__month").'
+                                         % (base_instance._meta.object_name, self.name,
+                                            _field_name))
+
                     parts = ['year', 'month', 'day']
                     try:
                         granularity = parts.index(inner) + 1
@@ -230,24 +264,21 @@ class AutoSlugField(SlugField):
                     else:
                         for part in parts[:granularity]:
                             lookup = '%s__%s' % (field_name, part)
-                            yield (lookup, getattr(value, part))
+                            yield lookup, getattr(value, part)
                 else:
                     if inner:
-                        raise ValueError('Wrong value "%s__%s" for constraint'
-                                         ' `unique_with` in %s.%s. Compound field'
-                                         ' names are only accepted for DateField'
-                                         ' attributes (such as "date__month").'
-                                         % (field_name, inner,
-                                            instance._meta.object_name, self.name))
-                    yield (field_name, value)
-        lookups = tuple(_get_lookups(instance))
+                        for res in _get_lookups(value, [inner]):
+                            yield _field_name, res[1]
+                    else:
+                        yield field_name, value
+
+        lookups = tuple(_get_lookups(instance, self.unique_with))
         model = instance.__class__
         field_name = self.name
         index = 1
         if self.max_length < len(slug):
             slug = slug[:self.max_length]
         orig_slug = slug
-        sep = SLUG_INDEX_SEPARATOR
         # keep changing the slug until it is unique
         while True:
             try:
@@ -259,12 +290,13 @@ class AutoSlugField(SlugField):
                 # the slug is not unique; change once more
                 index += 1
                 # ensure the resulting string is not too long
-                tail_length = len(sep) + len(str(index))
+                tail_length = len(self.index_sep) + len(str(index))
                 combined_length = len(orig_slug) + tail_length
                 if self.max_length < combined_length:
                     orig_slug = orig_slug[:self.max_length - tail_length]
                 # re-generate the slug
-                slug = '%s%s%d' % (orig_slug, sep, index)
+                data = dict(slug=orig_slug, sep=self.index_sep, index=index)
+                slug = '%(slug)s%(sep)s%(index)d' % data
             except model.DoesNotExist:
                 # slug is unique, no model uses it
                 return slug
